/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/core/v3/protocol.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_CONFIG_CORE_V3_PROTOCOL_PROTO_UPB_MINITABLE_H_
#define ENVOY_CONFIG_CORE_V3_PROTOCOL_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__core__v3__TcpProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__QuicKeepAliveSettings_msg_init;
extern const upb_MiniTable envoy__config__core__v3__QuicProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__UpstreamHttpProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__AlternateProtocolsCacheOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__AlternateProtocolsCacheOptions__AlternateProtocolsCacheEntry_msg_init;
extern const upb_MiniTable envoy__config__core__v3__HttpProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Http1ProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Http1ProtocolOptions__HeaderKeyFormat_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Http1ProtocolOptions__HeaderKeyFormat__ProperCaseWords_msg_init;
extern const upb_MiniTable envoy__config__core__v3__KeepaliveSettings_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Http2ProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Http2ProtocolOptions__SettingsParameter_msg_init;
extern const upb_MiniTable envoy__config__core__v3__GrpcProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Http3ProtocolOptions_msg_init;
extern const upb_MiniTable envoy__config__core__v3__SchemeHeaderTransformation_msg_init;

extern const upb_MiniTableFile envoy_config_core_v3_protocol_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_CORE_V3_PROTOCOL_PROTO_UPB_MINITABLE_H_ */
