/* This file was generated by upb_generator from the input file:
 *
 *     envoy/type/http/v3/path_transformation.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_TYPE_HTTP_V3_PATH_TRANSFORMATION_PROTO_UPB_MINITABLE_H_
#define ENVOY_TYPE_HTTP_V3_PATH_TRANSFORMATION_PROTO_UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__type__http__v3__PathTransformation_msg_init;
extern const upb_MiniTable envoy__type__http__v3__PathTransformation__Operation_msg_init;
extern const upb_MiniTable envoy__type__http__v3__PathTransformation__Operation__NormalizePathRFC3986_msg_init;
extern const upb_MiniTable envoy__type__http__v3__PathTransformation__Operation__MergeSlashes_msg_init;

extern const upb_MiniTableFile envoy_type_http_v3_path_transformation_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_TYPE_HTTP_V3_PATH_TRANSFORMATION_PROTO_UPB_MINITABLE_H_ */
